package com.akamai.analytics.akamaimedia;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.WindowManager;
import android.widget.ProgressBar;

import com.akamai.analytics.R;
import com.akamai.media.VideoPlayerContainer;
import com.akamai.media.VideoPlayerView;
import com.akamai.media.elements.MediaResource;
import com.akamai.uimobile.media.MediaPlayerController;

public class MainActivity extends Activity
        implements VideoPlayerContainer.VideoPlayerContainerCallback {

    private final static String TAG = "AMPv6";
    private final static String LICENSE = "vqKTzjO0G9MCPX9acxE533JgjzItfJH-DF9kd8QkaD24sIEudSfT4tzniSwxTFJj-MwUUALLOTt5DEstYhJSlg==";
    private final static String VIDEO_URL = "http://sblancovodmdev-f.akamaihd.net/i/temp/bbb.mp4/master.m3u8";


    private VideoPlayerContainer videoPlayerContainer;
    private VideoPlayerView videoPlayerView;
    private ProgressBar progressBar;
    private MediaPlayerController mMediaPlayerController;

    private MediaResource resource;
    private MediaAnalyticsTracker mediaAnalyticsTracker;

    private boolean mWasPlaying = false;
    private int mCurrentPosition = 0;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        mMediaPlayerController = (MediaPlayerController) findViewById(R.id.playerControls);
        videoPlayerContainer = (VideoPlayerContainer) findViewById(R.id.playerViewCtrl);
        videoPlayerContainer.addVideoPlayerContainerCallback(this);
        videoPlayerContainer.prepareResource(VIDEO_URL);
    }

    @Override
    public void onResourceReady(MediaResource mediaResource) {
        resource = mediaResource;

        videoPlayerView = videoPlayerContainer.getVideoPlayer();
        videoPlayerView.setProgressBarControl(progressBar);
        videoPlayerView.setLicense(LICENSE);
        videoPlayerView.setFullScreen(true);
        videoPlayerView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_KEEP_ASPECT_RATIO);
        videoPlayerView.setLogEnabled(true);
        mMediaPlayerController.setVideoPlayerView(videoPlayerContainer);

        mediaAnalyticsTracker = new MediaAnalyticsTracker(this);
        mediaAnalyticsTracker.setVideoPlayerView(videoPlayerView);
        mediaAnalyticsTracker.setMediaAnalyticsConfigUrl("http://ma1-r.analytics.edgesuite.net/config/beacon-3018.xml?setVideoObject=1");

        videoPlayerView.play(resource, mCurrentPosition);
    }

    @Override
    public void onVideoPlayerCreated() {
        Log.i(TAG, "onVideoPlayerCreated()");
    }

    @Override
    public void onResourceError() {
        Log.e(TAG, "onResourceError()");
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume");

        if (mWasPlaying) {
            videoPlayerView.resume();
            videoPlayerView.onResume();
            mWasPlaying = false;
        }
        super.onResume();
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause");

        boolean shouldStop = (videoPlayerView != null) && (videoPlayerView.isPlaying() || videoPlayerView.isPaused());
        if (shouldStop) {
            mWasPlaying = true;
            videoPlayerView.pause();
            videoPlayerView.onPause();
        }
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy");
        if(videoPlayerView != null){
            videoPlayerView.stop();
            videoPlayerView.onDestroy();
        }
        super.onDestroy();
    }

}
